package com.hai.schedule;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hai.common.utils.DateUtil;
import com.hai.common.utils.IDGenerator;
import com.hai.config.RechargeConfig;
import com.hai.entity.HighChildOrder;
import com.hai.entity.HighOrder;
import com.hai.entity.OutRechargeChildOrder;
import com.hai.entity.OutRechargeOrder;
import com.hai.order.service.OrderService;
import com.hai.service.OutRechargeChildOrderService;
import com.hai.service.OutRechargeOrderService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * @serviceName HighRechargeOrderSchedule.java
 * @author Sum1Dream
 * @version 1.0.0
 * @Description // 充值订单定时任务
 * @createTime 09:57 2023/1/13
 **/
@Configuration
public class HighRechargeOrderSchedule {

    Logger log = LoggerFactory.getLogger(HighRechargeOrderSchedule.class);

    @Resource
    private OutRechargeOrderService outRechargeOrderService;

    @Resource
    private OutRechargeChildOrderService rechargeChildOrderService;

    @Resource
    private OrderService orderService;

    /**
     * @return void
     * @Author Sum1Dream
     * @name cancelOrder.java
     * @Description // 定时发起
     * @Date 14:18 2022/5/31
     * @Param []
     */
    @Scheduled(cron = "0 0/5 * * * ?")   //每5分钟执行一次
    public void initRechargeOrder() {
        Map<String, Object> map = new HashMap<>();
        map.put("payStatus", String.valueOf(102));
        map.put("rechargeStatus", String.valueOf(205));
        List<OutRechargeOrder> orderList = outRechargeOrderService.getListRechargeOrder(map);
        if (orderList.size() > 0) {
            for (OutRechargeOrder order : orderList) {
                try {

                    // 定义小时数
                    Date rechargeTime = order.getCreateTimed();
                    Date currentTime = new Date();
                    int hours = (int) ((currentTime.getTime() - rechargeTime.getTime()) / (1000 * 60 * 60));

                    // 查询当前订单是否超时 如果超过72小时则处理退款
                    if (hours < 72) {
                        outRechargeOrderService.pollRequest(order);
                    } else {
                        // 话费订单处理
                        outRechargeOrderService.rechargeOrderHandle(order.getOrderNo());
                    }

                } catch (Exception e) {
                    log.error("HighCouponSchedule --> expiredCoupon() error!", e);
                }
            }
        }
        System.out.println("话费定时任务 -> 结束任务");
    }


}
