package com.hai.openApi.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.entity.*;
import com.hai.openApi.config.PriceComputeConfig;
import com.hai.openApi.service.ApiMchProductService;
import com.hai.openApi.service.ApiOrderCreateHandleService;
import com.hai.service.ApiMerchantsService;
import com.hai.service.HighGasOilPriceService;
import com.hai.service.HighMerchantStoreService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Service("apiOrderCreateHandleService")
public class ApiOrderCreateHandleServiceImpl implements ApiOrderCreateHandleService {

    @Resource
    private ApiMerchantsService apiMerchantsService;

    @Resource
    private HighGasOilPriceService highGasOilPriceService;

    @Resource
    private PriceComputeConfig priceComputeConfig;

    @Resource
    private HighMerchantStoreService highMerchantStoreService;

    @Resource
    private ApiMchProductService apiMchProductService;

    @Override
    public ApiOrder oilHandle(JSONObject object) throws Exception {

        //  获取下单内容
        JSONObject dataObject = JSONObject.parseObject(object.getString("content"));

        // 查询油枪价
        HighGasOilPrice highGasOilPrice =  highGasOilPriceService.getGasOilPriceByStoreAndOilNo(dataObject.getLong("storeCode") , dataObject.getString("gasOilNo"));

        // 查询api商户
        ApiMerchants apiMerchants = apiMerchantsService.findByMchId(object.getString("merchId"));

        // 查询加油门店
        HighMerchantStore merchantStore = highMerchantStoreService.getMerchantStoreById(dataObject.getLong("storeCode"));

        // 查询折扣比例
        Map<String , Object> mchProductMap = new HashMap<>();
        mchProductMap.put("productType" , object.getString("productType"));
        mchProductMap.put("mchId" , object.getString("merchId"));
        mchProductMap.put("merchantId" , merchantStore.getMerchantId());
        ApiMchProduct mchProduct = apiMchProductService.findByMap(mchProductMap);

        BigDecimal price = dataObject.getBigDecimal("refuelPrice").multiply(mchProduct.getDiscount()).divide(new BigDecimal(100)).setScale(2, BigDecimal.ROUND_HALF_DOWN);

        // 判断金额是否充足
        if (apiMerchants.getAmounts().compareTo(price) < 0) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.INSUFFICIENT_BALANCE);
        }

        // 组装内容
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("oilNo" , highGasOilPrice.getOilNo());
        jsonObject.put("oilNoName" , highGasOilPrice.getOilNoName());
        jsonObject.put("priceGun" , highGasOilPrice.getPriceGun());
        jsonObject.put("priceCost" , highGasOilPrice.getPriceGun().multiply(mchProduct.getDiscount()).divide(new BigDecimal(100)).setScale(2, BigDecimal.ROUND_HALF_UP));
        jsonObject.put("oilTypeName" , highGasOilPrice.getOilTypeName());
        jsonObject.put("storeName" , merchantStore.getStoreName());
        jsonObject.put("address" , merchantStore.getAddress());
        jsonObject.put("regionName" , merchantStore.getRegionName());

        // 组装api订单
        ApiOrder apiOrder = new ApiOrder();
        apiOrder.setCreateTime(new Date());
        apiOrder.setTitle(apiMerchants.getMerchantName() + "|在线加油｜扣款" + price);
        apiOrder.setFacePrice(dataObject.getBigDecimal("refuelPrice"));
        apiOrder.setCostPrice(price);
        apiOrder.setMchId(object.getString("merchId"));
        apiOrder.setMchName(apiMerchants.getMerchantName());
        apiOrder.setUpdateTime(new Date());
        apiOrder.setProductType(object.getInteger("productType"));
        apiOrder.setMchOrderNo(object.getString("orderNo"));
        apiOrder.setOrderPrice(dataObject.getBigDecimal("refuelPrice"));
        apiOrder.setNotifyUrl(object.getString("notifyUrl"));
        apiOrder.setContent(jsonObject.toJSONString());

        return apiOrder;
    }
}
