package com.hai.openApi.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.DateUtil;
import com.hai.dao.ApiOrderMapper;
import com.hai.entity.ApiMerchants;
import com.hai.entity.ApiOrder;
import com.hai.entity.ApiOrderExample;
import com.hai.entity.HighOrder;
import com.hai.openApi.service.ApiOrderCreateHandleService;
import com.hai.openApi.service.ApiOrderService;
import com.hai.order.model.CreateOrderChildModel;
import com.hai.order.model.CreateOrderModel;
import com.hai.order.service.OrderPayBeforeService;
import com.hai.order.service.OrderPayService;
import com.hai.order.service.OrderService;
import com.hai.order.type.OrderChildGoodsType;
import com.hai.order.type.OrderProductType;
import com.hai.order.type.OrderStatus;
import com.hai.order.utils.OrderUtil;
import com.hai.service.ApiMerchantsService;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

@Service("apiOrderService")
public class ApiOrderServiceImpl implements ApiOrderService {

    @Resource
    private ApiMerchantsService apiMerchantsService;

    @Resource
    private ApiOrderMapper apiOrderMapper;

    @Resource
    private ApiOrderCreateHandleService orderCreateHandleService;

    @Resource
    private OrderService orderService;

    @Resource
    private OrderPayBeforeService orderPayBeforeService;

    @Resource
    private OrderPayService orderPayService;

    @Resource
    private ApiOrderService apiOrderService;

    @Override
    @Transactional(rollbackFor=Exception.class,isolation = Isolation.SERIALIZABLE,propagation= Propagation.REQUIRES_NEW)
    public ApiOrder createOrder(JSONObject object) throws Exception {

        ApiOrder apiOrder = new ApiOrder();


        //  获取下单内容
        JSONObject dataObject = JSONObject.parseObject(object.getString("content"));


        CreateOrderModel createOrderModel = new CreateOrderModel();
        createOrderModel.setChildOrderList(new ArrayList<>());

        // 查询订单
        Map<String , Object> orderMap = new HashMap<>();
        orderMap.put("mchOrderNo" , object.getString("orderNo"));
        List<ApiOrder> apiOrderList = getOrderByMap(orderMap);

        ApiMerchants apiMerchants = apiMerchantsService.findByMchId(object.getString("merchId"));

        createOrderModel.setMemName(apiMerchants.getMerchantName());
        createOrderModel.setMemPhone(apiMerchants.getPhone());
        createOrderModel.setCompanyId(2L);
        createOrderModel.setOrderNo(OrderUtil.generateOrderNo());

        // 判断订单号是否重复
        if (apiOrderList.size() > 0) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.ORDER_ALREADY_EXISTS);
        }

        // 处理加油订单
        if (OrderProductType.PRODUCT_TYPE6.getNumber().equals(object.getInteger("productType"))) {
            apiOrder = orderCreateHandleService.oilHandle(object);

            // 组装子订单内容
            CreateOrderChildModel createOrderChildModel = new CreateOrderChildModel();
            createOrderChildModel.setMemName(apiMerchants.getMerchantName());
            createOrderChildModel.setMemPhone(apiMerchants.getPhone());
            createOrderChildModel.setCompanyId(2L);
            createOrderChildModel.setGoodsType(OrderChildGoodsType.TYPE3.getNumber());
            createOrderChildModel.setGasGunNo(dataObject.getString("gasGunNo"));
            createOrderChildModel.setGasOilNo(dataObject.getString("gasOilNo"));
            createOrderChildModel.setStoreId(dataObject.getLong("storeCode"));
            createOrderChildModel.setGoodsId(dataObject.getLong("storeCode"));
            createOrderChildModel.setSaleCount(1);
            createOrderChildModel.setIsTyAgent(false);
            createOrderChildModel.setGoodsPrice(apiOrder.getOrderPrice());

            // 组装订单
            createOrderModel.getChildOrderList().add(createOrderChildModel);
            orderService.createOrder(createOrderModel);
        }
        apiOrder.setOrderNo(createOrderModel.getOrderNo());
        apiOrder.setStatus(1);
        apiOrderMapper.insert(apiOrder);

        return apiOrder;

    }


    @Override
    @Transactional(rollbackFor=Exception.class,isolation = Isolation.SERIALIZABLE,propagation= Propagation.REQUIRES_NEW)
    public ApiOrder payOrder(JSONObject object) {

        ApiMerchants apiMerchants = apiMerchantsService.findByMchId(object.getString("merchId"));

        ApiOrder apiOrder = apiOrderService.findByOrderNo(object.getString("mchOrderNo") , null);

        // 查询订单
        HighOrder order = orderService.getOrderDetailByNo(object.getString("orderNo"));
        if(!order.getOrderStatus().equals(OrderStatus.STATUS1.getNumber())) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.ORDER_NO_STAY_PAY, "");
        }

        // 支付业务交易
        orderPayBeforeService.payOrderCheck(order);

        // 处理订单
        HighOrder highOrder = orderPayService.apiMchAmount(order , apiMerchants , apiOrder);

        if (highOrder.getOrderStatus() != OrderStatus.STATUS1.getNumber()) {
            apiOrder.setStatus(highOrder.getOrderStatus());
            apiOrder.setUpdateTime(new Date());
            apiOrderMapper.updateByPrimaryKey(apiOrder);
        }

        return apiOrder;

    }

    @Override
    public List<ApiOrder> getOrderByMap(Map<String, Object> map) {

        ApiOrderExample example = new ApiOrderExample();
        ApiOrderExample.Criteria criteria = example.createCriteria();


        if (MapUtils.getInteger(map, "productType") != null) {
            criteria.andProductTypeEqualTo(MapUtils.getInteger(map, "productType"));
        }
        if (StringUtils.isNotBlank(MapUtils.getString(map, "orderNo"))) {
            criteria.andOrderNoEqualTo(MapUtils.getString(map, "orderNo"));
        }
        if (StringUtils.isNotBlank(MapUtils.getString(map, "mchOrderNo"))) {
            criteria.andMchOrderNoEqualTo(MapUtils.getString(map, "mchOrderNo"));
        }
        if (StringUtils.isNotBlank(MapUtils.getString(map, "merchId"))) {
            criteria.andMchIdEqualTo(MapUtils.getString(map, "merchId"));
        }
        if (StringUtils.isNotBlank(MapUtils.getString(map, "createTimeS")) && StringUtils.isNotBlank(MapUtils.getString(map, "createTimeE"))) {
            criteria.andCreateTimeBetween(
                    DateUtil.format(MapUtils.getString(map, "createTimeS"), "yyyy-MM-dd HH:mm:ss"),
                    DateUtil.format(MapUtils.getString(map, "createTimeE"), "yyyy-MM-dd HH:mm:ss"));
        }

        return apiOrderMapper.selectByExample(example);
    }


    @Override
    public ApiOrder findByOrderNo(String mchOrderNo , String orderNo) {
        ApiOrderExample example = new ApiOrderExample();
        ApiOrderExample.Criteria criteria = example.createCriteria();

        if (mchOrderNo != null) {
            criteria.andMchOrderNoEqualTo(mchOrderNo);
        }

        if (orderNo != null) {
            criteria.andOrderNoEqualTo(orderNo);
        }

        List<ApiOrder> list = apiOrderMapper.selectByExample(example);

        if (list.size() > 0) {
            return list.get(0);
        }

        return null;
    }
}
