package com.hai.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;

import com.hai.common.security.AESEncodeUtil;
import com.hai.common.utils.*;
import com.hai.config.*;
import com.hai.dao.HighGasOrderPushMapper;
import com.hai.dao.OutRechargeOrderMapper;
import com.hai.dao.OutRechargeOrderMapperExt;
import com.hai.entity.*;
import com.hai.enum_type.*;
import com.hai.model.*;
import com.hai.order.service.OrderService;
import com.hai.order.type.OrderChildStatus;
import com.hai.order.type.OrderPayType;
import com.hai.order.type.OrderRefundOpUserType;
import com.hai.service.*;
import com.hai.service.pay.impl.GoodsOrderServiceImpl;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;


@Service("outRechargeOrderService")
public class OutRechargeOrderServiceImpl implements OutRechargeOrderService {

    private static final Logger log = LoggerFactory.getLogger(OutRechargeOrderServiceImpl.class);

    @Resource
    private OutRechargeOrderMapper outRechargeOrderMapper;

    @Resource
    private OutRechargeOrderMapperExt outRechargeOrderMapperExt;

    @Resource
    private OrderService orderService;

    @Resource
    private HighGasOrderPushMapper highGasOrderPushMapper;

    @Resource
    private HighUserService highUserService;

    @Resource
    private OutRechargeOrderService outRechargeOrderService;

    @Resource
    private OutRechargePriceService outRechargePriceService;

    @Resource
    private GoodsOrderServiceImpl goodsOrderService;

    @Resource
    private BsIntegralRebateService bsIntegralRebateService;

    @Resource
    private HighDiscountUserRelService highDiscountUserRelService;

    @Resource
    private HighDiscountAgentCodeService highDiscountAgentCodeService;

    @Resource
    private OutRechargeChildOrderService outRechargeChildOrderService;

    @Resource
    private BsRequestRecordService bsRequestRecordService;

    @Resource
    private BsConfigService bsConfigService;

    @Resource
    private SecConfigService secConfigService;

    @Resource
    private HighUserPayPasswordService highUserPayPasswordService;

    @Resource
    private ApiMerchantsService apiMerchantsService;

    @Resource
    private ApiAmountRecordService apiAmountRecordService;

    @Resource
    private ApiOrderRecordService apiOrderRecordService;

    @Resource
    private BsRechargeCodeService rechargeCodeService;

    @Resource
    private HighOrderService highOrderService;

    @Override
    public List<OutRechargeOrder> getListRechargeOrder(Map<String, Object> map) {
        OutRechargeOrderExample example = new OutRechargeOrderExample();
        OutRechargeOrderExample.Criteria criteria = example.createCriteria();

        if (MapUtils.getLong(map, "userId") != null) {
            criteria.andUserIdEqualTo(MapUtils.getLong(map, "userId"));
        }
        if (MapUtils.getLong(map, "companyId") != null) {
            criteria.andCompanyIdEqualTo(MapUtils.getLong(map, "companyId"));
        }
        if (MapUtils.getInteger(map, "payType") != null) {
            criteria.andPayTypeEqualTo(MapUtils.getInteger(map, "payType"));
        }
        if (MapUtils.getInteger(map, "laborUnionCard") != null) {
            criteria.andLaborUnionCardEqualTo(MapUtils.getString(map, "laborUnionCard"));
        }
        if (MapUtils.getInteger(map, "payStatus") != null) {
            criteria.andPayStatusEqualTo(MapUtils.getInteger(map, "payStatus"));
        }
        if (MapUtils.getInteger(map, "rechargeType") != null) {
            criteria.andRechargeTypeEqualTo(MapUtils.getInteger(map, "rechargeType"));
        }
        if (MapUtils.getString(map, "orderNo") != null) {
            criteria.andOrderNoLike("%" + MapUtils.getString(map, "orderNo") + "%");
        }
        if (MapUtils.getString(map, "userPhone") != null) {
            criteria.andUserPhoneEqualTo(MapUtils.getString(map, "userPhone"));
        }
        if (MapUtils.getString(map, "rechargeContent") != null) {
            criteria.andRechargeContentLike("%" + MapUtils.getString(map, "rechargeContent") + "%");
        }
        if (MapUtils.getInteger(map, "operatorType") != null) {
            criteria.andOperatorTypeEqualTo(MapUtils.getInteger(map, "operatorType"));
        }
        if (MapUtils.getString(map, "merchId") != null) {
            criteria.andMerchIdEqualTo(MapUtils.getString(map, "merchId"));
        }

        if (MapUtils.getInteger(map, "rechargeStatus") != null) {
            criteria.andRechargeStatusEqualTo(MapUtils.getInteger(map, "rechargeStatus"));
        }
        if (StringUtils.isNotBlank(MapUtils.getString(map, "payTimeS")) && StringUtils.isNotBlank(MapUtils.getString(map, "payTimeE"))) {
            criteria.andPayTimeBetween(
                    DateUtil.format(MapUtils.getString(map, "payTimeS"), "yyyy-MM-dd HH:mm:ss"),
                    DateUtil.format(MapUtils.getString(map, "payTimeE"), "yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(MapUtils.getString(map, "createTimeS")) && StringUtils.isNotBlank(MapUtils.getString(map, "createTimeE"))) {
            criteria.andCreateTimedBetween(
                    DateUtil.format(MapUtils.getString(map, "createTimeS"), "yyyy-MM-dd HH:mm:ss"),
                    DateUtil.format(MapUtils.getString(map, "createTimeE"), "yyyy-MM-dd HH:mm:ss"));
        }

        if (MapUtils.getBoolean(map , "whetherTimeOut") != null && MapUtils.getBoolean(map , "whetherTimeOut")) {
            // 获取当前时间
            Date date= new Date();
            Calendar calendar = new GregorianCalendar();
            calendar.setTime(date);
            // 把日期往前减少三天,整数  往后推,负数往前移动
            calendar.add(Calendar.DATE, -3);
            // 这个时间就是日期往后推一天的结果
            date = calendar.getTime();
            criteria.andCreateTimedLessThan(date);
        }


        example.setOrderByClause("create_timed desc");
        return outRechargeOrderMapper.selectByExample(example);
    }

    @Override
    public OutRechargeOrder findByOrderId(Long orderId) {
        return outRechargeOrderMapper.selectByPrimaryKey(orderId);
    }

    @Override
    public OutRechargeOrder findByOrderNo(String orderNo) {

        OutRechargeOrderExample example = new OutRechargeOrderExample();
        example.createCriteria().andOrderNoEqualTo(orderNo);

        List<OutRechargeOrder> list = outRechargeOrderMapper.selectByExample(example);

        if (list.size() == 0) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未知的订单号");
        }

        OutRechargeOrder outRechargeOrder = list.get(0);

        // 充值子订单
        outRechargeOrder.setOutRechargeChildOrders(outRechargeChildOrderService.getListRechargeChildOrderByOrderId(outRechargeOrder.getId()));

        return outRechargeOrder;
    }

    // todo 接口已经废弃
    @Override
    @Transactional(
            isolation = Isolation.SERIALIZABLE,
            propagation = Propagation.REQUIRES_NEW)
    public OutRechargeOrder insertOrder(JSONObject object) throws Exception {

        OutRechargeOrder outRechargeOrder = new OutRechargeOrder();

        HighUser user = highUserService.findByUserId(object.getLong("userId"));

        // 产品id
        Long goodsId = object.getLong("goodsId");

        // 充值内容
        String rechargeContent = object.getString("rechargeContent");

        // 查询产品详情
        OutRechargePriceModel outRechargePrice = outRechargePriceService.findById(goodsId, null);

        Map<String, Object> listMap = new HashMap<>();
        listMap.put("productType", "3");
        listMap.put("returnType", 1);
        listMap.put("sourceId", goodsId);

        // 查询产品积分抵扣比例
        BsProductDiscount bsProductDiscount = bsConfigService.getProductDiscountByMap(listMap);

        // 判断充值系统是否关闭
        if (!secConfigService.isConfig("RECHARGE", "1")) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.RECHARGE_CLOSE, "");
        }

        HighDiscountUserRel highDiscountUserRel = null;
        // 判断是否有优惠券
        if (object.getLong("memDiscountId") != null) {
            outRechargeOrder.setMemDiscountId(object.getLong("memDiscountId"));
            // 卡优惠券信息
            highDiscountUserRel = highDiscountUserRelService.getRelById(object.getLong("memDiscountId"));
            if (highDiscountUserRel == null || highDiscountUserRel.getStatus() != 1) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "优惠券状态错误");
            }
            if (!highDiscountUserRel.getHighDiscount().getUseScope().equals(DiscountUseScope.type1.getType())
                    && !highDiscountUserRel.getHighDiscount().getUseScope().equals(DiscountUseScope.type3.getType())) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法使用此优惠券");
            }
        } else {
            outRechargeOrder.setPayRealPrice(outRechargePrice.getPayPrice());
        }

        // 优惠券抵扣
        if (highDiscountUserRel != null) {

            outRechargeOrder.setMemDiscountName(highDiscountUserRel.getHighDiscount().getDiscountName());
            BigDecimal payPrice = new BigDecimal(0);

            // 卡卷类型  1：满减  2：抵扣   3：折扣
            if (highDiscountUserRel.getHighDiscount().getDiscountType() == 1) {
                // 如果商品支付总额 小于 满减价格
                if (outRechargePrice.getRechargePrice().compareTo(highDiscountUserRel.getHighDiscount().getDiscountCondition()) < 0) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.UN_MEMBER_ERROR, "订单未达到" + highDiscountUserRel.getHighDiscount().getDiscountCondition() + "元,无法使用此优惠券");
                }
                // 计算支付金额 = 商品充值总额 - 满减额度
                payPrice = outRechargePrice.getRechargePrice().subtract(highDiscountUserRel.getHighDiscount().getDiscountPrice());
                outRechargeOrder.setDiscountDeductionPrice(highDiscountUserRel.getHighDiscount().getDiscountPrice());

            }

            // 卡卷类型  1：满减  2：抵扣   3：折扣
            if (highDiscountUserRel.getHighDiscount().getDiscountType() == 2) {
                // 计算支付金额 = 商品充值总额 - 满减额度
                payPrice = outRechargePrice.getRechargePrice().subtract(highDiscountUserRel.getHighDiscount().getDiscountPrice());
                outRechargeOrder.setDiscountDeductionPrice(highDiscountUserRel.getHighDiscount().getDiscountPrice());

            }

            // 卡卷类型  1：满减  2：抵扣   3：折扣
            if (highDiscountUserRel.getHighDiscount().getDiscountType() == 3) {
                BigDecimal discountPrice = highDiscountUserRel.getHighDiscount().getDiscountPrice();
                // 订单总额 * 折扣
                payPrice = outRechargePrice.getRechargePrice().multiply(discountPrice);
                outRechargeOrder.setDiscountDeductionPrice(outRechargePrice.getRechargePrice().subtract(payPrice));
                outRechargeOrder.setPayRealPrice(payPrice);
            }

            // 如果总额小于0
            if (payPrice.compareTo(new BigDecimal("0")) < 0) {
                outRechargeOrder.setPayRealPrice(new BigDecimal("0"));
            } else {
                outRechargeOrder.setPayRealPrice(payPrice);
            }
        }

        // 判断积分数量是否大于0
        if (object.getLong("integralNum") > 0 && bsProductDiscount.getDiscount().compareTo(new BigDecimal("0")) > 0) {

            // 判断用户积分是否够
            if (object.getLong("integralNum") > user.getGold()) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "用户积分不足");
            }

            // 积分抵扣金额
            BigDecimal integralDeductionPrice = object.getBigDecimal("integralNum").divide(new BigDecimal(100).setScale(2, RoundingMode.HALF_UP));
            // 最高可抵扣金额
            BigDecimal maxIntegralDeductionPrice = outRechargeOrder.getPayRealPrice().multiply(bsProductDiscount.getDiscount()).divide(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP);
            // 判读积分是否大于限制额度
            if (maxIntegralDeductionPrice.compareTo(integralDeductionPrice) < 0) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "订单最大抵扣积分金额：" + maxIntegralDeductionPrice);
            }

            // 判断积分抵扣比例是否为100% 并且积分数量是否可以抵扣最后的支付金额
            if (bsProductDiscount.getDiscount().compareTo(new BigDecimal(100)) == 0 && integralDeductionPrice.compareTo(outRechargeOrder.getPayRealPrice()) == 0) {
                // 查询用户支付密码
                HighUserPayPassword userPayPassword = highUserPayPasswordService.getDetailByUser(object.getLong("userId"));
                if (userPayPassword == null) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.NOT_SET_USER_PAY_PWD, "");
                }
                if (StringUtils.isBlank(object.getString("password"))) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.NOT_ENTER_USER_PAY_PWD, "");
                }
                // 校验支付密码
                if (!AESEncodeUtil.aesEncrypt(object.getString("password")).equals(userPayPassword.getPassword())) {
                    throw ErrorHelp.genException(SysCode.System, ErrorCode.USER_PAY_PWD_ERROR, "");
                }
            }

            outRechargeOrder.setIntegralDeductionPrice(integralDeductionPrice);
            outRechargeOrder.setPayRealPrice(outRechargeOrder.getPayRealPrice().subtract(outRechargeOrder.getIntegralDeductionPrice()));

        }

        outRechargeOrder.setUserId(user.getId());
        outRechargeOrder.setIntegralNum(object.getBigDecimal("integralNum"));
        outRechargeOrder.setRechargeContent(rechargeContent);
        outRechargeOrder.setCompanyId(outRechargePrice.getCompanyId());
        outRechargeOrder.setUserName(user.getName());
        outRechargeOrder.setRechargeType(outRechargePrice.getRechargeType());
        outRechargeOrder.setUserPhone(user.getPhone());
        outRechargeOrder.setOrderNo("RCG" + DateUtil.date2String(new Date(), "yyyyMMddHHmmss") + IDGenerator.nextId(5));
        outRechargeOrder.setCreateTimed(new Date());
        outRechargeOrder.setGoodsId(goodsId);
        outRechargeOrder.setWhetherRebate(false);
        outRechargeOrder.setMerchId(object.getString("mchId"));
        outRechargeOrder.setRechargePrice(outRechargePrice.getRechargePrice());
        if (object.getLong("memDiscountId") != null) {
            outRechargeOrder.setPayPrice(outRechargePrice.getRechargePrice());
        } else {
            outRechargeOrder.setPayPrice(outRechargePrice.getPayPrice());
        }
        outRechargeOrder.setType(1);
        outRechargeOrder.setOrderPrice(outRechargePrice.getRechargePrice());
        outRechargeOrder.setOperatorName(OperatorEnum.getNameByType(outRechargePrice.getOperatorType()));
        outRechargeOrder.setOperatorType(outRechargePrice.getOperatorType());


        // 判断积分支付是否扣完金额
        if (outRechargeOrder.getPayRealPrice().compareTo(new BigDecimal(0)) == 0) {
            // 201:充值中 202：充值成功 203：充值失败 204：未充值
            outRechargeOrder.setRechargeStatus(204);
            // 101.待支付 102.已支付 100.已完成 104.已取消 105.已退款
            outRechargeOrder.setPayStatus(102);
            outRechargeOrder.setPayTime(new Date());
            outRechargeOrder.setPaySerialNo("22" + DateUtil.date2String(new Date(), "yyyyMMddHHmmss") + IDGenerator.nextId(5));
        } else {
            // 201:充值中 202：充值成功 203：充值失败 204：未充值
            outRechargeOrder.setRechargeStatus(204);
            // 101.待支付 102.已支付 100.已完成 104.已取消 105.已退款
            outRechargeOrder.setPayStatus(101);
        }

        // 使用优惠券
        if (outRechargeOrder.getMemDiscountId() != null) {
            HighDiscountUserRel discountUserRel = highDiscountUserRelService.getRelById(outRechargeOrder.getMemDiscountId());
            discountUserRel.setUseTime(new Date()); // 使用时间
            discountUserRel.setStatus(2);  //状态  0:已过期  1：未使用  2：已使用
            highDiscountUserRelService.updateDiscountUserRel(discountUserRel);

            HighDiscountAgentCode code = highDiscountAgentCodeService.getCodeById(discountUserRel.getDiscountAgentCodeId());
            code.setStatus(3);
            highDiscountAgentCodeService.updateCode(code);
        }



        outRechargeOrderMapper.insert(outRechargeOrder);

        if (outRechargeOrder.getIntegralNum() != null) {
            String remark = "话费订单" + outRechargeOrder.getOrderNo() + "下单，使用积分：" +  object.getInteger("integralNum");
            highUserService.goldHandle(outRechargeOrder.getUserId(), outRechargeOrder.getIntegralNum().intValue(), 2, 2, outRechargeOrder.getId() , remark);
        }

        if (outRechargeOrder.getPayStatus() == 102) {
            pollRequest(outRechargeOrder);
        }

        return outRechargeOrder;

    }

    @Override
    public void updateOrder(OutRechargeOrder outRechargeOrder) {
        outRechargeOrderMapper.updateByPrimaryKey(outRechargeOrder);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void cancelOrder(Long orderId) {
        OutRechargeOrder order = findByOrderId(orderId);
        if (order != null && order.getPayStatus() == 101) {
            // 101.待支付 102.已支付 100.已完成 104.已取消 105.已退款
            order.setPayStatus(104);
            order.setCancelTime(new Date());

            if (order.getMemDiscountId() != null) {
                HighDiscountUserRel rel = highDiscountUserRelService.getRelById(order.getMemDiscountId());
                if (rel != null) {
                    rel.setStatus(1);  // 状态  0:已过期  1：未使用  2：已使用
                    rel.setUseTime(null);
                    highDiscountUserRelService.updateDiscountUserRel(rel);

                    HighDiscountAgentCode code = highDiscountAgentCodeService.getCodeById(rel.getDiscountAgentCodeId());
                    code.setStatus(2);
                    highDiscountAgentCodeService.updateCode(code);
                }
            }

            if (order.getIntegralNum() != null) {
                // 积分返回
                String remark = "话费订单" + order.getOrderNo() + "取消，退还积分：" +  order.getIntegralNum().intValue();
                highUserService.goldHandle(order.getUserId(), order.getIntegralNum().intValue(), 1, 3, order.getId() , remark);
            }

            updateOrder(order);
        }

    }

    @Override
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void finishOrder(Long orderId) {
        OutRechargeOrder order = findByOrderId(orderId);
        if (order != null) {
            order.setRechargeStatus(202); // 订单状态：1 待支付  2 已支付  3.已完成  4. 已取消  5.已退款
            order.setPayStatus(100); // 订单状态：1 待支付  2 已支付  3.已完成  4. 已取消  5.已退款
            order.setFinishTime(new Date());
            updateOrder(order);
        }
    }

    @Override
    public List<OutRechargeOrderModel> getOrderCount(Map<String, Object> map) throws Exception {


        if (MapUtils.getLong(map, "createTimeS") != null) {
            map.put("createTimeS", MapUtils.getString(map, "createTimeS"));
        }

        if (MapUtils.getLong(map, "createTimeE") != null) {
            map.put("createTimeE", MapUtils.getString(map, "createTimeE"));
        }

        if (MapUtils.getLong(map, "payTimeS") != null) {
            map.put("payTimeS", MapUtils.getString(map, "payTimeS"));
        }

        if (MapUtils.getLong(map, "payTimeE") != null) {
            map.put("payTimeE", MapUtils.getString(map, "payTimeE"));
        }

        return outRechargeOrderMapper.selectOrderCount(map);
    }


    @Override
    public List<OutOrderModel> getListOrderCount(Map<String, String> map) throws Exception {
        String finishTimeS = map.get("finishTimeS");
        String finishTimeE = map.get("finishTimeE");
        Integer status;
        if (StringUtils.isNotBlank(map.get("status"))) {
            status = Integer.valueOf(map.get("status"));
        } else {
            status = null;
        }
        String fTimeS;
        String fTimeE;
        if (StringUtils.isNotBlank(finishTimeS)) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(new Date(Long.parseLong(finishTimeS)));
            //时
            calendar.set(Calendar.HOUR_OF_DAY, 00);
            //分
            calendar.set(Calendar.MINUTE, 00);
            //秒
            calendar.set(Calendar.SECOND, 00);
            fTimeS = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(calendar.getTime());

        } else {
            fTimeS = "2010-09-13 22:36:01";
        }
        if (StringUtils.isNotBlank(finishTimeE)) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(new Date(Long.parseLong(finishTimeE)));
            //时
            calendar.set(Calendar.HOUR_OF_DAY, 00);
            //分
            calendar.set(Calendar.MINUTE, 00);
            //秒
            calendar.set(Calendar.SECOND, 00);
            fTimeE = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(calendar.getTime());
        } else {
            fTimeE = DateUtil.date2String(new Date(), "yyyy-MM-dd HH:mm:ss");
        }
        return outRechargeOrderMapperExt.getListOrderCount(fTimeS, fTimeE, status);
    }

    @Override
    public List<OutUserOrderModel> getUserCountList(Map<String, String> map) throws Exception {
        String finishTimeS = map.get("finishTimeS");
        String finishTimeE = map.get("finishTimeE");
        String fTimeS;
        String fTimeE;
        if (StringUtils.isNotBlank(finishTimeS)) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(new Date(Long.parseLong(finishTimeS)));
            //时
            calendar.set(Calendar.HOUR_OF_DAY, 00);
            //分
            calendar.set(Calendar.MINUTE, 00);
            //秒
            calendar.set(Calendar.SECOND, 00);
            fTimeS = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(calendar.getTime());

        } else {
            fTimeS = "2010-09-13 22:36:01";
        }
        if (StringUtils.isNotBlank(finishTimeE)) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(new Date(Long.parseLong(finishTimeE)));
            //时
            calendar.set(Calendar.HOUR_OF_DAY, 00);
            //分
            calendar.set(Calendar.MINUTE, 00);
            //秒
            calendar.set(Calendar.SECOND, 00);
            fTimeE = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(calendar.getTime());
        } else {
            fTimeE = DateUtil.date2String(new Date(), "yyyy-MM-dd HH:mm:ss");
        }
        return outRechargeOrderMapperExt.getListUserCount(fTimeS, fTimeE);
    }

    @Override
    public OrderCountModel rechargeOrderByIndex(Integer code) {
        return outRechargeOrderMapper.rechargeOrderByIndex(code);
    }

    @Override
    public void rechargeOrderToRefund(Long orderId) throws Exception {

    }

    @Override
    @Transactional(
            propagation= Propagation.REQUIRES_NEW,
            isolation = Isolation.READ_COMMITTED,
            timeout = 25,
            rollbackFor = Exception.class)
    public void refundOrder(Long orderId , OutRechargeChildOrder childOrder) throws Exception {
        OutRechargeOrder outRechargeOrder = outRechargeOrderService.findByOrderId(orderId);

        // 101.待支付 102.已支付 100.已完成 104.已取消 105.已退款
        if (outRechargeOrder.getPayStatus() != 102) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法退款，订单不处于已支付状态");
        }

        //  充值状态：201:充值中 202：充值成功 203：充值失败 204:未充值
        if (outRechargeOrder.getRechargeStatus() == 202) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "无法退款，订单已充值成功无法退款z");
        }

        // 订单处理
        HighOrder highOrder = highOrderService.getOrderByOrderNo(outRechargeOrder.getOrderNo());
        orderService.refundOrder(highOrder, "话费自动退款",OrderRefundOpUserType.TYPE1,
                null,
                "系统自动退款");

        // 话费充值业务订单处理
        outRechargeOrder.setPayStatus(105);
        outRechargeOrder.setRechargeStatus(203);
        outRechargeOrder.setRefundTime(new Date());
        outRechargeOrderService.updateOrder(outRechargeOrder);

        // 充值订单记录修改状态
        childOrder.setStatus(103);
        outRechargeChildOrderService.updateOrder(childOrder);
    }


    @Override
    public void pollRequest(OutRechargeOrder outRechargeOrder) throws Exception {

        // 查询充值产品
        OutRechargePriceModel outRechargePrice = outRechargePriceService.findById(outRechargeOrder.getGoodsId(), null);

        // 查询充值子订单列表
        // 定义子订单请求map
        Map<String, Object> childOrderMap = new HashMap<>();

        // 组装查询子订单数据
        childOrderMap.put("parent_order_id", outRechargeOrder.getId());
        // 查询子订单
        List<OutRechargeChildOrder> childOrderList = outRechargeChildOrderService.getListRechargeChildOrder(childOrderMap);

        // 定义新的子订单对象
        OutRechargeChildOrder childOrder = new OutRechargeChildOrder();

        // 初始化充值平台 1:尖椒 2：龙阅 3：城宇 4.畅停
        int platformType = 0;

        // 判断子订单是否存在充值中订单
        if (childOrderList.size() > 0) {
            // 修改之前的自订单充值记录
            childOrder = childOrderList.get(0);

            // -1 取消，0充值中 ，1充值成功，2充值失败，3部分成功 99 未配置平台查询方法
            Integer status = getRechargeOrderLegality(outRechargeOrder , childOrder);
            if (status == 0 || status == 3 || status == 99) {
                return;
            } else if (status == 1) {
                completeOrder(outRechargeOrder.getOrderNo());
            }

            // 修改充值记录状态
            childOrder.setStatus(103);
            childOrder.setUpdateTime(new Date());
            outRechargeChildOrderService.updateOrder(childOrder);
        }

        // 获取充值平台
        platformType = getRechargePlatformType();

        // 定义下单请求数据对象
        JSONObject object = new JSONObject();

        // 定义新充值订单号
        String orderNo = "CZ" + DateUtil.date2String(new Date(), "yyyyMMddHHmmss") + IDGenerator.nextId(5);

        // 查询平台
        String string = outRechargePrice.getRechargePlatform().substring(1,outRechargePrice.getRechargePlatform().length()-1);
        String[] rechargePlatform = string.split("-");

        // 判断平台只定义一个那么类型就是唯一
        if (string.length() == 1) {
            platformType = Integer.parseInt(string);
        } else {
            // 如果平台充值记录暂无 则充值第一个
            if (platformType == 0) {
                platformType = Integer.parseInt(rechargePlatform[0]);
            } else {
                // 获取上一条充值平台的下一个平台内容
                platformType = Integer.parseInt(rechargePlatform[ArrayUtils.arrayIndex(rechargePlatform , String.valueOf(platformType))]);
            }

        }

        // 定义查询充值产品编码
        Map<String, Object> map = new HashMap<>();
        map.put("type" , 1);

        for (String s : rechargePlatform) {
            // 尖椒充值
            if (s.equals("1") && platformType == 1) {
                object.put("out_order_id", orderNo);
                object.put("amount", outRechargePrice.getRechargePrice());
                object.put("mobile", outRechargeOrder.getRechargeContent());
                object.put("notifyUrl", CommonSysConst.getSysConfig().getJjNotifyUrl());
                JSONObject returnObject = RechargeConfig.rechargeOrderByJj(object);
                object.put("return_content", returnObject);
                if (returnObject.getLong("code") == 200) {
                    childOrder.setStatus(102);
                } else {
                    childOrder.setStatus(103);
                }
                childOrder.setRechargePlatform(1);
                break;
            }
            // 龙阅充值
            if (s.equals("2") && platformType == 2) {

                map.put("sourceId" , outRechargePrice.getId());
                map.put("rechargePlatform" , platformType);
                BsRechargeCode rechargeCode = rechargeCodeService.findByMap(map);
                object.put("out_trade_num", orderNo);
                object.put("product_id", rechargeCode.getGoodsId());
                object.put("mobile", outRechargeOrder.getRechargeContent());
                object.put("notifyUrl", CommonSysConst.getSysConfig().getLyNotifyUrl());
                JSONObject returnObject = RechargeConfig.rechargeOrderByLy(object);
                object.put("return_content", returnObject);
                if (returnObject.getLong("errno") == 0) {
                    childOrder.setStatus(102);
                } else {
                    childOrder.setStatus(103);
                }
                childOrder.setRechargePlatform(2);
                break;
            }
            // 城宇充值
            if (s.equals("3") && platformType == 3) {

                map.put("sourceId" , outRechargePrice.getId());
                map.put("rechargePlatform" , platformType);
                BsRechargeCode rechargeCode = rechargeCodeService.findByMap(map);
                object.put("agentOrderId", orderNo);
                object.put("productId", rechargeCode.getGoodsId());
                object.put("mobile", outRechargeOrder.getRechargeContent());
                JSONObject returnObject = RechargeConfig.rechargeOrderByCy(object);
                object.put("return_content", returnObject);
                if (returnObject.getBoolean("success")) {
                    childOrder.setStatus(102);
                } else {
                    childOrder.setStatus(103);
                }
                childOrder.setRechargePlatform(3);
                break;
            }
            // 畅停充值
            if (s.equals("4") && platformType == 4) {

                map.put("sourceId" , outRechargePrice.getId());
                map.put("rechargePlatform" , platformType);
                BsRechargeCode rechargeCode = rechargeCodeService.findByMap(map);
                object.put("orderid", orderNo);
                object.put("product_id", rechargeCode.getGoodsId());
                object.put("phone", outRechargeOrder.getRechargeContent());
                object.put("price", outRechargeOrder.getRechargePrice());
                JSONObject returnObject = RechargeConfig.rechargeOrderByHg(object);
                object.put("return_content", returnObject);
                if (returnObject.getLong("code") != null &&  returnObject.getLong("code") == 200) {
                    childOrder.setStatus(102);
                } else {
                    childOrder.setStatus(103);
                }
                childOrder.setRechargePlatform(4);
                break;
            }
        }

        childOrder.setOrderNo(orderNo);
        childOrder.setCreateTime(new Date());
        childOrder.setParentOrderId(outRechargeOrder.getId());
        childOrder.setUpdateTime(new Date());
        outRechargeChildOrderService.insertOrder(childOrder);

        // 判断是否充值提交成功
        if (childOrder.getStatus() == 102) {
            outRechargeOrder.setRechargeStatus(201);
            outRechargeOrder.setPayStatus(102);
        }

        if (childOrder.getStatus() == 103) {
            outRechargeOrder.setRechargeStatus(205);
            outRechargeOrder.setPayStatus(102);
        }
        outRechargeOrderService.updateOrder(outRechargeOrder);
    }

    @Override
    public void completeOrder(String orderNo) {

        OutRechargeChildOrder childOrder = outRechargeChildOrderService.findByOrderNo(orderNo);
        // 查询充值订单
        OutRechargeOrder order = outRechargeOrderService.findByOrderId(childOrder.getParentOrderId());

        // 修改话费充值业务订单状态
        childOrder.setStatus(101);
        order.setRechargeStatus(202);
        order.setPayStatus(100);
        order.setFinishTime(new Date());
        outRechargeOrderService.updateOrder(order);
        outRechargeChildOrderService.updateOrder(childOrder);

        // 修改订单状态
        HighOrder highOrder = highOrderService.getOrderByOrderNo(order.getOrderNo());
        if (highOrder != null) {
            highOrder.setOrderStatus(OrderStatusEnum.type3.getType());
            highOrder.setFinishTime(new Date());
            for (HighChildOrder highChildOrder : highOrder.getHighChildOrderList()) {
                highChildOrder.setChildOrderStatus(OrderChildStatus.STATUS3.getNumber());
                highChildOrder.setFinishTime(new Date());
            }
            orderService.updateOrderData(highOrder);
        }


        // 推送小程序消息
        new Thread(() -> {
            HighUser highUser = highUserService.findByUserId(highOrder.getMemId());
            WxMsgConfig.rechargedSuccess(
                    order.getRechargeContent() + "充值成功",
                    String.valueOf(order.getPayRealPrice()),
                    order.getOrderNo(),
                    order.getFinishTime(),
                    OrderPayType.getDataByType(order.getPayType()).getName(),
                    highUser.getOpenId());
        }).start();

    }


    @Override
    public void editOutRechargeOrder(HighOrder orderData) {
        OutRechargeOrder outRechargeOrder = outRechargeOrderService.findByOrderNo(orderData.getOrderNo());

        // 201:充值中 202：充值成功 203：充值失败 204：未充值
        outRechargeOrder.setRechargeStatus(201);
        // 101.待支付 102.已支付 100.已完成 104.已取消 105.已退款
        outRechargeOrder.setPayStatus(102);
        outRechargeOrder.setMemDiscountId(orderData.getMemDiscountId());
        outRechargeOrder.setMemDiscountName(orderData.getMemDiscountName());
        outRechargeOrder.setPayType(orderData.getPayType());
        outRechargeOrder.setLaborUnionCard(orderData.getMemCardNo());
        outRechargeOrder.setDiscountDeductionPrice(orderData.getDeductionCouponPrice());
        outRechargeOrder.setIntegralDeductionPrice( new BigDecimal(orderData.getPayGold().toString()).divide(new BigDecimal(100)));
        outRechargeOrder.setIntegralNum(new BigDecimal(orderData.getPayGold()));
        outRechargeOrder.setPayPrice(orderData.getPayPrice());
        outRechargeOrder.setPayRealPrice(orderData.getPayRealPrice());
        outRechargeOrderService.updateOrder(outRechargeOrder);
    }
    @Override
    public List<ApiRechargePhoneModel> getApiRechargePhoneByList(Map<String, Object> map) {
        return outRechargeOrderMapper.getApiRechargePhoneByList(map);
    }

    @Override
    public void rechargeOrderHandle(String orderNo) throws Exception {

        // 定义话费充值业务订单模型 or 充值记录订单模型
        OutRechargeOrder outRechargeOrder = findByOrderNo(orderNo);
        OutRechargeChildOrder childOrder = outRechargeOrder.getOutRechargeChildOrders().get(0);

        // 状态需要支付状态为已充值 并且 充值状态不为 充值成功
        if (outRechargeOrder.getPayStatus() == 102 && outRechargeOrder.getRechargeStatus() != 202) {

            // 判断充值记录是否存在
            if (outRechargeOrder.getOutRechargeChildOrders().size() == 0) {

                //
                OutRechargeChildOrder outRechargeChildOrder = new OutRechargeChildOrder();
                outRechargeChildOrder.setOrderNo("CZ" + DateUtil.date2String(new Date(), "yyyyMMddHHmmss") + IDGenerator.nextId(5));
                outRechargeChildOrder.setParentOrderId(outRechargeOrder.getId());
                outRechargeChildOrder.setUpdateTime(new Date());
                outRechargeChildOrder.setCreateTime(new Date());
                outRechargeChildOrderService.insertOrder(outRechargeChildOrder);
                // 话费退款
                refundOrder(outRechargeOrder.getId() , outRechargeChildOrder);
            } else {
                // -1 取消，0充值中 ，1充值成功，2充值失败，3部分成功
                Integer status = getRechargeOrderLegality(outRechargeOrder , childOrder);
                if (status == 2 || status == -1) {
                    refundOrder(outRechargeOrder.getId() , childOrder);
                } else if (status == 1) {
                    completeOrder(outRechargeOrder.getOrderNo());
                }
            }

        }
    }


    @Override
    public Integer getRechargeOrderLegality(OutRechargeOrder outRechargeOrder, OutRechargeChildOrder childOrder) throws Exception {
        JSONObject object = new JSONObject();
        // 龙阅
        if (childOrder.getRechargePlatform() == 2) {

            object.put("out_trade_nums" , childOrder.getOrderNo());

            JSONObject jsonObject = RechargeConfig.checkOrderByLy(object);

            JSONArray objectJSONArray = jsonObject.getJSONArray("data");
            JSONObject objectData = new JSONObject();
            for (Object objects : objectJSONArray) {
                objectData =  (JSONObject) objects;
            }
            if (objectData.size() == 0) {
                if (jsonObject.getInteger("errno") == 0) {
                    return 2;
                } else {
                    return 99;
                }
            } else {
                // -1取消，0充值中 ，1充值成功，2充值失败，3部分成功
                return  objectData.getInteger("state");
            }

        }
        // 畅停
        if (childOrder.getRechargePlatform() == 4) {
            object.put("orderid" , childOrder.getOrderNo());

            JSONObject jsonObject = RechargeConfig.checkOrderByHg(object);
            if (jsonObject.getJSONArray("data") == null) {
                return 99;
            } else {
                JSONArray objectJSONArray = jsonObject.getJSONArray("data");
                JSONObject objectData;
                for (Object objects : objectJSONArray) {
                    objectData =  (JSONObject) objects;
                    // 0充值中，1成功，2失败，3已取消，4部分充值，5缓存订单
                    // -1取消，0充值中 ，1充值成功，2充值失败，3部分成功

                    if (objectData.getInteger("status") == 0 || objectData.getInteger("status") == 5) {
                        return 0;
                    } else if (objectData.getInteger("status") == 1) {
                        return 1;
                    } else if (objectData.getInteger("status") == 2) {
                        return 2;
                    } else if (objectData.getInteger("status") == 3) {
                        return -1;
                    } else if (objectData.getInteger("status") == 4) {
                        return 3;
                    }

                }
            }
        }
        return 99;
    }

    private Integer getRechargePlatformType() {
        OutRechargeChildOrder outRechargeChildOrder = outRechargeChildOrderService.findOutRechargeChildOrderByDesc();

        if (outRechargeChildOrder != null) {
            return outRechargeChildOrder.getRechargePlatform();
        }
        return  0;
    }
}
